# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-12 13:56
from __future__ import unicode_literals

import demoproject.demoapp.models
from django.db import migrations, models
import strategy_field.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DemoAllModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('choice', strategy_field.fields.StrategyClassField()),
                ('multiple', strategy_field.fields.MultipleStrategyClassField()),
                ('custom', strategy_field.fields.StrategyField()),
                ('custom_multiple', strategy_field.fields.MultipleStrategyField()),
            ],
        ),
        migrations.CreateModel(
            name='DemoCallableModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.StrategyClassField()),
            ],
        ),
        migrations.CreateModel(
            name='DemoCustomModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.StrategyField()),
            ],
        ),
        migrations.CreateModel(
            name='DemoModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.StrategyClassField()),
            ],
        ),
        migrations.CreateModel(
            name='DemoModelCallableDefault',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.StrategyClassField(default=demoproject.demoapp.models.cc, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DemoModelContext',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='DemoModelDefault',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.StrategyClassField(default=b'demoproject.demoapp.models.Sender1', null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DemoModelNone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.StrategyClassField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DemoModelNoRegistry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('klass', strategy_field.fields.StrategyClassField(blank=True, null=True)),
                ('instance', strategy_field.fields.StrategyField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DemoMultipleCustomModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.MultipleStrategyField()),
            ],
        ),
        migrations.CreateModel(
            name='DemoMultipleModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', strategy_field.fields.MultipleStrategyClassField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DemoModelProxy',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('demoapp.demomodel',),
        ),
    ]
